from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0002_auto_20150606_2003'),
    ]

    operations = [
        migrations.CreateModel(
            name='ThumbnailOption',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('width', models.IntegerField(help_text='width in pixel.', verbose_name='width')),
                ('height', models.IntegerField(help_text='height in pixel.', verbose_name='height')),
                ('crop', models.BooleanField(default=True, verbose_name='crop')),
                ('upscale', models.BooleanField(default=True, verbose_name='upscale')),
            ],
            options={
                'ordering': ('width', 'height'),
                'verbose_name': 'thumbnail option',
                'verbose_name_plural': 'thumbnail options',
            },
        ),
    ]



import logging
import warnings

from django import forms
from django.contrib.admin.sites import site
from django.contrib.admin.widgets import ForeignKeyRawIdWidget
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.http import urlencode
from django.utils.safestring import mark_safe

from .. import settings as filer_settings
from ..models import File
from ..settings import ICON_CSS_LIB
from ..utils.compatibility import truncate_words
from ..utils.model_label import get_model_label


logger = logging.getLogger(__name__)


class AdminFileWidget(ForeignKeyRawIdWidget):
    choices = None

    def render(self, name, value, attrs=None, renderer=None):
        obj = self.obj_for_value(value)
        css_id = attrs.get("id", "id_image_x")
        related_url = None
        change_url = ""
        if value:
            try:
                file_obj = File.objects.get(pk=value)
                related_url = (
                    file_obj.logical_folder.get_admin_directory_listing_url_path()
                )
                change_url = file_obj.get_admin_change_url()
            except Exception as e:
                # catch exception and manage it. We can re-raise it for debugging
                # purposes and/or just logging it, provided user configured
                # proper logging configuration
                if filer_settings.FILER_ENABLE_LOGGING:
                    logger.error("Error while rendering file widget: %s", e)
                if filer_settings.FILER_DEBUG:
                    raise
        if not related_url:
            related_url = reverse("admin:filer-directory_listing-last")
        params = self.url_parameters()
        params["_pick"] = "file"
        if params:
            lookup_url = "?" + urlencode(sorted(params.items()))
        else:
            lookup_url = ""
        if "class" not in attrs:
            # The JavaScript looks for this hook.
            attrs["class"] = "vForeignKeyRawIdAdminField"
        # rendering the super for ForeignKeyRawIdWidget on purpose here because
        # we only need the input and none of the other stuff that
        # ForeignKeyRawIdWidget adds
        hidden_input = super(ForeignKeyRawIdWidget, self).render(
            name, value, attrs
        )  # grandparent super
        context = {
            "hidden_input": hidden_input,
            "lookup_url": f"{related_url}{lookup_url}",
            "change_url": change_url,
            "object": obj,
            "lookup_name": name,
            "id": css_id,
            "admin_icon_delete": ("admin/img/icon-deletelink.svg"),
        }
        html = render_to_string("admin/filer/widgets/admin_file.html", context)
        return mark_safe(html)

    def label_for_value(self, value):
        obj = self.obj_for_value(value)
        return "&nbsp;<strong>%s</strong>" % truncate_words(obj, 14)

    def obj_for_value(self, value):
        if value:
            try:
                # the next line may never bee reached
                key = self.rel.get_related_field().name
                obj = self.rel.model._default_manager.get(**{key: value})
            except ObjectDoesNotExist:
                obj = None
        else:
            obj = None
        return obj

    class Media:
        css = {
            "all": ("filer/css/admin_filer.css",) + ICON_CSS_LIB,
        }
        js = ("filer/js/dist/admin-file-widget.bundle.js",)


class AdminFileFormField(forms.ModelChoiceField):
    widget = AdminFileWidget

    def __init__(self, rel, queryset, to_field_name, *args, **kwargs):
        self.rel = rel
        self.queryset = queryset
        self.to_field_name = to_field_name
        self.max_value = None
        self.min_value = None
        kwargs.pop("widget", None)
        super().__init__(queryset, widget=self.widget(rel, site), *args, **kwargs)

    def widget_attrs(self, widget):
        widget.required = self.required
        return {}


class FilerFileField(models.ForeignKey):
    default_form_class = AdminFileFormField
    default_model_class = File

    def __init__(self, **kwargs):
        to = kwargs.pop("to", None)
        dfl = get_model_label(self.default_model_class)
        if to and get_model_label(to).lower() != dfl.lower():
            msg = "In {}: ForeignKey must point to {}; instead passed {}"
            warnings.warn(msg.format(self.__class__.__name__, dfl, to), SyntaxWarning)
        kwargs["to"] = dfl  # hard-code `to` to model `filer.File`
        super().__init__(**kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "form_class": self.default_form_class,
            "rel": self.remote_field,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)



# Generated by Django 4.1.9 on 2023-06-15 20:03

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
        ('filer', '0014_folder_permission_choices'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='owned_%(class)ss', to=settings.AUTH_USER_MODEL, verbose_name='owner'),
        ),
        migrations.AlterField(
            model_name='file',
            name='polymorphic_ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_%(app_label)s.%(class)s_set+', to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='image',
            name='file_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='%(app_label)s_%(class)s_file', serialize=False, to='filer.file'),
        ),
    ]



import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0009_auto_20171220_1635'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image',
            name='file_ptr',
            field=models.OneToOneField(primary_key=True, serialize=False, related_name='filer_image_file', parent_link=True, to='filer.File', on_delete=django.db.models.deletion.CASCADE, auto_created=False),
        ),
    ]



import json
import os
import tempfile
from io import StringIO

from django.conf import settings
from django.core.files import File as DjangoFile
from django.core.management import call_command
from django.test import TestCase

from filer import settings as filer_settings
from filer.models import Folder
from filer.models.filemodels import File
from filer.settings import FILER_IMAGE_MODEL
from filer.utils.loader import load_model
from tests.helpers import SettingsOverride, create_folder_structure, create_image, create_superuser


Image = load_model(FILER_IMAGE_MODEL)


class DumpDataTests(TestCase):

    def setUp(self):
        self.superuser = create_superuser()
        self.img = create_image()
        self.image_name = 'test_file.jpg'
        self.filename = os.path.join(settings.FILE_UPLOAD_TEMP_DIR, self.image_name)
        self.img.save(self.filename, 'JPEG')

    def tearDown(self):
        os.remove(self.filename)
        for f in File.objects.all():
            f.delete()
        pass

    def create_filer_image(self, folder=None):
        with open(self.filename, 'rb') as file:
            file_obj = DjangoFile(file, name=self.image_name)
            image = Image.objects.create(owner=self.superuser,
                                         original_filename=self.image_name,
                                         file=file_obj, folder=folder)
        return image

    def create_filer_file(self, folder=None):
        with open(self.filename, 'rb') as file:
            file_obj = DjangoFile(file, name=self.image_name)
            fileobj = File.objects.create(owner=self.superuser,
                                          original_filename=self.image_name,
                                          file=file_obj, folder=folder)
        return fileobj

    def test_dump_data_base(self):
        """
        Testing the case of dump full and empty dataset
        """
        fileobj = self.create_filer_file()
        jdata, jdata2 = StringIO(), StringIO()
        call_command("dumpdata", "filer", stdout=jdata)
        fileobj.delete()
        call_command("dumpdata", "filer", stdout=jdata2)
        data = json.loads(jdata.getvalue())
        data2 = json.loads(jdata2.getvalue())
        self.assertEqual(len(data), 1)
        self.assertEqual(len(data2), 0)

    def test_dump_load_data(self):
        """
        Testing the dump / load with no dump of file content data
        """
        # Initialize the test data
        create_folder_structure(1, 1)
        fileobj = self.create_filer_file(Folder.objects.all()[0])

        self.assertEqual(Image.objects.count(), 0)
        image = self.create_filer_image()
        image.save()
        image_size = image._width, image._height
        self.assertEqual(Image.objects.count(), 1)

        jdata = StringIO()

        # Dump the current data
        fobj = tempfile.NamedTemporaryFile(suffix=".json", delete=False)
        call_command("dumpdata", "filer", stdout=jdata, indent=3)

        # Delete database and filesystem data
        complete = os.path.join(fileobj.file.storage.location, fileobj.path)
        os.unlink(complete)
        fileobj.delete()

        # Dump data to json file
        fobj.write(jdata.getvalue().encode('utf-8'))
        fobj.seek(0)

        # Load data back
        call_command("loaddata", fobj.name, stdout=jdata)

        # Database data is restored
        self.assertEqual(Folder.objects.all().count(), 1)
        self.assertEqual(File.objects.all().count(), 2)
        self.assertEqual(File.objects.all()[0].original_filename, self.image_name)
        self.assertEqual(Image.objects.count(), 1)

        fileobj = File.objects.all()[0]
        image = Image.objects.all()[0]
        self.assertEqual(image._width, image_size[0])
        self.assertEqual(image._height, image_size[1])

        complete = os.path.join(fileobj.file.storage.location, fileobj.path)
        # Filesystem data is not
        self.assertFalse(os.path.exists(complete))

    def test_dump_load_data_content(self):
        """
        Testing the dump / load with full dump of file content data
        """
        with SettingsOverride(filer_settings, FILER_DUMP_PAYLOAD=True):
            # Initialize the test data
            create_folder_structure(1, 1)
            fileobj = self.create_filer_file(Folder.objects.all()[0])
            jdata = StringIO()

            # Dump the current data
            fobj = tempfile.NamedTemporaryFile(suffix=".json", delete=False)
            call_command("dumpdata", "filer", stdout=jdata, indent=3)

            # Delete database and filesystem data and
            complete = os.path.join(fileobj.file.storage.location, fileobj.path)
            os.unlink(complete)
            fileobj.delete()

            # Dump data to json file
            fobj.write(jdata.getvalue().encode('utf-8'))
            fobj.seek(0)

            # Load data back
            call_command("loaddata", fobj.name, stdout=jdata)

            # Database data is restored
            self.assertEqual(Folder.objects.all().count(), 1)
            self.assertEqual(File.objects.all().count(), 1)
            self.assertEqual(File.objects.all()[0].original_filename, self.image_name)

            fileobj = File.objects.all()[0]
            complete = os.path.join(fileobj.file.storage.location, fileobj.path)
            # Filesystem data too!
            self.assertTrue(os.path.exists(complete))



import warnings

from django.templatetags.static import static

from ..settings import FILER_ADMIN_ICON_SIZES


class IconsMixin:
    """
    Can be used on any model that has a _icon attribute. will return a dict
    containing urls for icons of different sizes with that name.
    """
    @property
    def icons(self):
        warnings.warn("Method 'icons' is deprecated and will be re moved in future versions",
                      DeprecationWarning)
        r = {}
        if getattr(self, '_icon', False):
            for size in FILER_ADMIN_ICON_SIZES:
                try:
                    r[size] = static("filer/icons/{}_{}x{}.png".format(
                        self._icon, size, size))
                except ValueError:
                    # Do not raise an exception while trying to call static()
                    # on non-existent icon file. This avoids the issue with
                    # rendering parts of the template as empty blocks that
                    # happens on an attempt to access object 'icons' attribute
                    # in template.
                    pass
        return r



import logging
import os

from django.conf import settings
from django.utils.module_loading import import_string as get_storage_class
from django.utils.translation import gettext_lazy as _

from .utils.loader import load_object
from .utils.recursive_dictionary import RecursiveDictionaryWithExcludes


logger = logging.getLogger(__name__)

# FILER_IMAGE_MODEL setting is used to swap Image model.
# If such global setting does not exist, it will be created at this point (with default model name).
# This is needed especially when using this setting in migrations.
if not hasattr(settings, 'FILER_IMAGE_MODEL'):
    setattr(settings, 'FILER_IMAGE_MODEL', 'filer.Image')
FILER_IMAGE_MODEL = settings.FILER_IMAGE_MODEL

FILER_DEBUG = getattr(settings, 'FILER_DEBUG', False)  # When True makes
FILER_SUBJECT_LOCATION_IMAGE_DEBUG = getattr(settings, 'FILER_SUBJECT_LOCATION_IMAGE_DEBUG', False)
FILER_WHITESPACE_COLOR = getattr(settings, 'FILER_WHITESPACE_COLOR', '#FFFFFF')

FILER_0_8_COMPATIBILITY_MODE = getattr(settings, 'FILER_0_8_COMPATIBILITY_MODE', False)

FILER_ENABLE_LOGGING = getattr(settings, 'FILER_ENABLE_LOGGING', False)
if FILER_ENABLE_LOGGING:
    FILER_ENABLE_LOGGING = (
        FILER_ENABLE_LOGGING and (getattr(settings, 'LOGGING')
                             and ('' in settings.LOGGING['loggers']
                             or 'filer' in settings.LOGGING['loggers'])))

FILER_ENABLE_PERMISSIONS = getattr(settings, 'FILER_ENABLE_PERMISSIONS', False)
FILER_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS = getattr(settings, 'FILER_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS', False)
FILER_IS_PUBLIC_DEFAULT = getattr(settings, 'FILER_IS_PUBLIC_DEFAULT', True)

FILER_PAGINATE_BY = getattr(settings, 'FILER_PAGINATE_BY', 100)

if hasattr(settings, "FILER_ADMIN_ICON_SIZES"):
    logger.warning("FILER_ADMIN_ICON_SIZES is deprecated and will be removed in the future.")

_ICON_SIZES = getattr(settings, 'FILER_ADMIN_ICON_SIZES', ('16', '32', '48', '64'))
# Reliably sort by integer value, but keep icon size as string.
# (There is some code in the wild that depends on this being strings.)
FILER_ADMIN_ICON_SIZES = [str(i) for i in sorted([int(s) for s in _ICON_SIZES])]

# Currently, these two icon sizes are hard-coded into the admin and admin templates
FILER_TABLE_ICON_SIZE = getattr(settings, "FILER_TABLE_ICON_SIZE", 40)
FILER_THUMBNAIL_ICON_SIZE = getattr(settings, "FILER_THUMBNAIL_ICON_SIZE", 120)
DEFERRED_THUMBNAIL_SIZES = (
    FILER_TABLE_ICON_SIZE,
    2 * FILER_TABLE_ICON_SIZE,
    FILER_THUMBNAIL_ICON_SIZE,
    2 * FILER_THUMBNAIL_ICON_SIZE,
)


# This is an ordered iterable that describes a list of
# classes that I should check for when adding files
FILER_FILE_MODELS = getattr(
    settings, 'FILER_FILE_MODELS',
    (FILER_IMAGE_MODEL, 'filer.File'))

if hasattr(settings, "STORAGES") and 'default' in settings.STORAGES:
    DEFAULT_FILE_STORAGE = settings.STORAGES['default'].get('BACKEND', 'django.core.files.storage.FileSystemStorage')
else:
    DEFAULT_FILE_STORAGE = getattr(settings, 'DEFAULT_FILE_STORAGE', 'django.core.files.storage.FileSystemStorage')

MINIMAL_FILER_STORAGES = {
    'public': {
        'main': {
            'ENGINE': None,
            'OPTIONS': {},
        },
        'thumbnails': {
            'ENGINE': None,
            'OPTIONS': {},
        }
    },
    'private': {
        'main': {
            'ENGINE': None,
            'OPTIONS': {},
        },
        'thumbnails': {
            'ENGINE': None,
            'OPTIONS': {},
        },
    },
}


DEFAULT_FILER_STORAGES = {
    'public': {
        'main': {
            'ENGINE': DEFAULT_FILE_STORAGE,
            'OPTIONS': {},
            'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
            'UPLOAD_TO_PREFIX': 'filer_public',
        },
        'thumbnails': {
            'ENGINE': DEFAULT_FILE_STORAGE,
            'OPTIONS': {},
            'THUMBNAIL_OPTIONS': {
                'base_dir': 'filer_public_thumbnails',
            },
        },
    },
    'private': {
        'main': {
            'ENGINE': 'filer.storage.PrivateFileSystemStorage',
            'OPTIONS': {
                'location': os.path.abspath(os.path.join(settings.MEDIA_ROOT, '../smedia/filer_private')),
                'base_url': '/smedia/filer_private/',
            },
            'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
            'UPLOAD_TO_PREFIX': '',
        },
        'thumbnails': {
            'ENGINE': 'filer.storage.PrivateFileSystemStorage',
            'OPTIONS': {
                'location': os.path.abspath(os.path.join(settings.MEDIA_ROOT, '../smedia/filer_private_thumbnails')),
                'base_url': '/smedia/filer_private_thumbnails/',
            },
            'THUMBNAIL_OPTIONS': {},
        },
    },
}

MINIMAL_FILER_SERVERS = {
    'private': {
        'main': {
            'ENGINE': None,
            'OPTIONS': {},
        },
        'thumbnails': {
            'ENGINE': None,
            'OPTIONS': {},
        },
    },
}

DEFAULT_FILER_SERVERS = {
    'private': {
        'main': {
            'ENGINE': 'filer.server.backends.default.DefaultServer',
            'OPTIONS': {},
        },
        'thumbnails': {
            'ENGINE': 'filer.server.backends.default.DefaultServer',
            'OPTIONS': {},
        },
    },
}

FILER_STORAGES = RecursiveDictionaryWithExcludes(MINIMAL_FILER_STORAGES, rec_excluded_keys=('OPTIONS', 'THUMBNAIL_OPTIONS'))
if FILER_0_8_COMPATIBILITY_MODE:
    user_filer_storages = {
        'public': {
            'main': {
                'ENGINE': DEFAULT_FILE_STORAGE,
                'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
                'UPLOAD_TO_PREFIX': getattr(settings, 'FILER_PUBLICMEDIA_PREFIX', 'filer_public'),
            },
            'thumbnails': {
                'ENGINE': DEFAULT_FILE_STORAGE,
                'OPTIONS': {},
                'THUMBNAIL_OPTIONS': {
                    'base_dir': 'filer_public_thumbnails',
                },
            },
        },
        'private': {
            'main': {
                'ENGINE': DEFAULT_FILE_STORAGE,
                'UPLOAD_TO': 'filer.utils.generate_filename.randomized',
                'UPLOAD_TO_PREFIX': getattr(settings, 'FILER_PRIVATEMEDIA_PREFIX', 'filer_private'),
            },
            'thumbnails': {
                'ENGINE': DEFAULT_FILE_STORAGE,
                'OPTIONS': {},
                'THUMBNAIL_OPTIONS': {
                    'base_dir': 'filer_private_thumbnails',
                },
            },
        },
    }
else:
    user_filer_storages = getattr(settings, 'FILER_STORAGES', {})

FILER_STORAGES.rec_update(user_filer_storages)


def update_storage_settings(user_settings, defaults, s, t):
    if not user_settings[s][t]['ENGINE']:
        user_settings[s][t]['ENGINE'] = defaults[s][t]['ENGINE']
        user_settings[s][t]['OPTIONS'] = defaults[s][t]['OPTIONS']
    if t == 'main':
        if 'UPLOAD_TO' not in user_settings[s][t]:
            user_settings[s][t]['UPLOAD_TO'] = defaults[s][t]['UPLOAD_TO']
        if 'UPLOAD_TO_PREFIX' not in user_settings[s][t]:
            user_settings[s][t]['UPLOAD_TO_PREFIX'] = defaults[s][t]['UPLOAD_TO_PREFIX']
    if t == 'thumbnails':
        if 'THUMBNAIL_OPTIONS' not in user_settings[s][t]:
            user_settings[s][t]['THUMBNAIL_OPTIONS'] = defaults[s][t]['THUMBNAIL_OPTIONS']
    return user_settings


update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'public', 'main')
update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'public', 'thumbnails')
update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'private', 'main')
update_storage_settings(FILER_STORAGES, DEFAULT_FILER_STORAGES, 'private', 'thumbnails')

FILER_SERVERS = RecursiveDictionaryWithExcludes(MINIMAL_FILER_SERVERS, rec_excluded_keys=('OPTIONS',))
FILER_SERVERS.rec_update(getattr(settings, 'FILER_SERVERS', {}))


def update_server_settings(settings, defaults, s, t):
    if not settings[s][t]['ENGINE']:
        settings[s][t]['ENGINE'] = defaults[s][t]['ENGINE']
        settings[s][t]['OPTIONS'] = defaults[s][t]['OPTIONS']
    return settings


update_server_settings(FILER_SERVERS, DEFAULT_FILER_SERVERS, 'private', 'main')
update_server_settings(FILER_SERVERS, DEFAULT_FILER_SERVERS, 'private', 'thumbnails')


# Public media (media accessible without any permission checks)
FILER_PUBLICMEDIA_STORAGE = get_storage_class(FILER_STORAGES['public']['main']['ENGINE'])(**FILER_STORAGES['public']['main']['OPTIONS'])
FILER_PUBLICMEDIA_UPLOAD_TO = load_object(FILER_STORAGES['public']['main']['UPLOAD_TO'])
if 'UPLOAD_TO_PREFIX' in FILER_STORAGES['public']['main']:
    FILER_PUBLICMEDIA_UPLOAD_TO = load_object('filer.utils.generate_filename.prefixed_factory')(FILER_PUBLICMEDIA_UPLOAD_TO, FILER_STORAGES['public']['main']['UPLOAD_TO_PREFIX'])
FILER_PUBLICMEDIA_THUMBNAIL_STORAGE = get_storage_class(FILER_STORAGES['public']['thumbnails']['ENGINE'])(**FILER_STORAGES['public']['thumbnails']['OPTIONS'])
FILER_PUBLICMEDIA_THUMBNAIL_OPTIONS = FILER_STORAGES['public']['thumbnails']['THUMBNAIL_OPTIONS']


# Private media (media accessible through permissions checks)
FILER_PRIVATEMEDIA_STORAGE = get_storage_class(FILER_STORAGES['private']['main']['ENGINE'])(**FILER_STORAGES['private']['main']['OPTIONS'])
FILER_PRIVATEMEDIA_UPLOAD_TO = load_object(FILER_STORAGES['private']['main']['UPLOAD_TO'])
if 'UPLOAD_TO_PREFIX' in FILER_STORAGES['private']['main']:
    FILER_PRIVATEMEDIA_UPLOAD_TO = load_object('filer.utils.generate_filename.prefixed_factory')(FILER_PRIVATEMEDIA_UPLOAD_TO, FILER_STORAGES['private']['main']['UPLOAD_TO_PREFIX'])
FILER_PRIVATEMEDIA_THUMBNAIL_STORAGE = get_storage_class(FILER_STORAGES['private']['thumbnails']['ENGINE'])(**FILER_STORAGES['private']['thumbnails']['OPTIONS'])
FILER_PRIVATEMEDIA_THUMBNAIL_OPTIONS = FILER_STORAGES['private']['thumbnails']['THUMBNAIL_OPTIONS']
FILER_PRIVATEMEDIA_SERVER = load_object(FILER_SERVERS['private']['main']['ENGINE'])(**FILER_SERVERS['private']['main']['OPTIONS'])
FILER_PRIVATEMEDIA_THUMBNAIL_SERVER = load_object(FILER_SERVERS['private']['thumbnails']['ENGINE'])(**FILER_SERVERS['private']['thumbnails']['OPTIONS'])

# By default limit number of simultaneous uploads if we are using SQLite
if settings.DATABASES['default']['ENGINE'].endswith('sqlite3'):
    _uploader_connections = 1
else:
    _uploader_connections = 3
FILER_UPLOADER_CONNECTIONS = getattr(
    settings, 'FILER_UPLOADER_CONNECTIONS', _uploader_connections)
FILER_UPLOADER_MAX_FILES = getattr(
    settings, 'FILER_UPLOADER_MAX_FILES', 100)
FILER_UPLOADER_MAX_FILE_SIZE = getattr(
    settings, 'FILER_UPLOADER_MAX_FILE_SIZE', None)


FILER_DUMP_PAYLOAD = getattr(settings, 'FILER_DUMP_PAYLOAD', False)  # Whether the filer shall dump the files payload

FILER_CANONICAL_URL = getattr(settings, 'FILER_CANONICAL_URL', 'canonical/')

TABLE_LIST_TYPE = 'tb'
THUMBNAIL_LIST_TYPE = 'th'
FILER_FOLDER_ADMIN_LIST_TYPE_CHOICES = (
    TABLE_LIST_TYPE,
    THUMBNAIL_LIST_TYPE,
)
FILER_FOLDER_ADMIN_DEFAULT_LIST_TYPE = getattr(settings, 'FILER_FOLDER_ADMIN_DEFAULT_LIST_TYPE', TABLE_LIST_TYPE)
if FILER_FOLDER_ADMIN_DEFAULT_LIST_TYPE not in FILER_FOLDER_ADMIN_LIST_TYPE_CHOICES:
    FILER_FOLDER_ADMIN_DEFAULT_LIST_TYPE = TABLE_LIST_TYPE

FILER_FOLDER_ADMIN_LIST_TYPE_SWITCHER_SETTINGS = {
    TABLE_LIST_TYPE: {
        'icon': 'th-list',
        'tooltip_text': _('Show table view'),
        'template': 'admin/filer/folder/directory_table_list.html',
    },
    THUMBNAIL_LIST_TYPE: {
        'icon': 'th-large',
        'tooltip_text': _('Show thumbnail view'),
        'template': 'admin/filer/folder/directory_thumbnail_list.html',
    },
}

IMAGE_EXTENSIONS = ['.jpg', '.jpeg', '.png', '.gif', '.webp']
IMAGE_MIME_TYPES = ['gif', 'jpeg', 'png', 'x-png', 'svg+xml', 'webp']

FILE_VALIDATORS = {
    "text/html": ["filer.validation.deny_html"],
    "image/svg+xml": ["filer.validation.validate_svg"],
    "application/octet-stream": ["filer.validation.deny"],
}

remove_mime_types = getattr(settings, "FILER_REMOVE_FILE_VALIDATORS", [])
for mime_type in remove_mime_types:  # pragma: no cover
    if mime_type in FILE_VALIDATORS:
        del FILE_VALIDATORS[mime_type]

for mime_type, validators in getattr(settings, "FILER_ADD_FILE_VALIDATORS", {}).items():  # pragma: no cover
    if mime_type in FILE_VALIDATORS:
        FILE_VALIDATORS[mime_type] += list(validators)
    else:
        FILE_VALIDATORS[mime_type] = list(validators)

FILER_MIME_TYPE_WHITELIST = getattr(settings, "FILER_MIME_TYPE_WHITELIST", [])


# Determine if django CMS is installed and if it comes with its own iconset

ICON_CSS_LIB = ("filer/css/admin_filer.fa.icons.css",)
if "cms" in settings.INSTALLED_APPS:  # pragma: no cover
    try:
        from cms import __version__
        from cms.utils.urlutils import static_with_version

        if __version__ >= "4":
            ICON_CSS_LIB = (
                static_with_version("cms/css/cms.admin.css"),
                "filer/css/admin_filer.cms.icons.css",
            )
    except (ModuleNotFoundError, ImportError):
        # Import error? No django CMS used: stay with own icons
        pass


# SVG are their own thumbnails if their size is below this limit
FILER_MAX_SVG_THUMBNAIL_SIZE = getattr(settings, "FILER_MAX_SVG_THUMBNAIL_SIZE", 1024 * 1024)  # 1MB default



import string

from django.test import TestCase
from filer.utils.files import get_valid_filename


class GetValidFilenameTest(TestCase):

    def setUp(self):
        """
        Set up the test case by reading the configuration settings for the maximum filename length.
        """
        self.max_length = 155
        self.random_suffix_length = 16

    def test_short_filename_remains_unchanged(self):
        """
        Test that a filename under the maximum length remains unchanged.
        """
        original = "example.jpg"
        result = get_valid_filename(original)
        self.assertEqual(result, "example.jpg")

    def test_long_filename_is_truncated_and_suffix_appended(self):
        """
        Test that a filename longer than the maximum allowed length is truncated and a random
        hexadecimal suffix of length 16 is appended, resulting in exactly 255 characters.
        """
        base = "a" * 300  # 300 characters base
        original = f"{base}.jpg"
        result = get_valid_filename(original)
        self.assertEqual(
            len(result),
            self.max_length,
            "Filename length should be exactly 255 characters."
        )
        # Verify that the last 16 characters form a valid hexadecimal string.
        random_suffix = result[-16:]
        valid_hex_chars = set(string.hexdigits)
        self.assertTrue(all(c in valid_hex_chars for c in random_suffix),
                        "The suffix is not a valid hexadecimal string.")

    def test_filename_with_extension_preserved(self):
        """
        Test that the file extension is preserved (and slugified) after processing.
        """
        original = "This is a test IMAGE.JPG"
        result = get_valid_filename(original)
        self.assertTrue(result.endswith(".jpg"),
                        "File extension was not preserved correctly.")

    def test_unicode_characters(self):
        """
        Test that filenames with Unicode characters are handled correctly.
        """
        original = "fiłęñâmé_üñîçødé.jpeg"
        result = get_valid_filename(original)
        self.assertTrue(result.endswith(".jpeg"),
                        "File extension is not preserved for unicode filename.")
        # Verify that the resulting filename contains only allowed characters.
        allowed_chars = set(string.ascii_lowercase + string.digits + "._-")
        for char in result:
            self.assertIn(char, allowed_chars,
                          f"Unexpected character '{char}' found in filename.")

    def test_edge_case_exact_length(self):
        """
        Test that a filename exactly at the maximum allowed length remains unchanged.
        """
        extension = ".png"
        base_length = 155 - len(extension)
        base = "b" * base_length
        original = f"{base}{extension}"
        result = get_valid_filename(original)
        self.assertEqual(
            len(result),
            self.max_length,
            "Filename with length exactly 255 should remain unchanged."
        )
        self.assertEqual(result, original,
                         "Filename with length exactly 255 should not be modified.")

    def test_edge_case_filenames(self):
        """
        Test filenames at various boundary conditions to ensure correct behavior.
        """
        max_length = self.max_length
        random_suffix_length = self.random_suffix_length
        extension = ".jpg"

        # Test case 1: Filename with length exactly max_length - 1.
        base_length = max_length - 1 - len(extension)
        base = "c" * base_length
        original = f"{base}{extension}"
        result = get_valid_filename(original)
        self.assertEqual(result, original,
                         "Filename with length max_length-1 should remain unchanged.")

        # Test case 2: Filename with length exactly equal to max_length - random_suffix_length.
        base_length = max_length - random_suffix_length - len(extension)
        base = "d" * base_length
        original = f"{base}{extension}"
        result = get_valid_filename(original)
        self.assertEqual(result, original,
                         "Filename with length equal to max_length - random_suffix_length should remain unchanged.")

        # Test case 3: Filename with length exactly equal to max_length - random_suffix_length - 1.
        base_length = max_length - random_suffix_length - 1 - len(extension)
        base = "e" * base_length
        original = f"{base}{extension}"
        result = get_valid_filename(original)
        self.assertEqual(result, original,
                         "Filename with length equal to max_length - random_suffix_length - 1 should remain unchanged.")



# Generated by Django 1.9.7 on 2016-06-23 20:27
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0005_auto_20160623_1425'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image',
            name='file_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='%(app_label)s_%(class)s_file', serialize=False, to='filer.File', parent_link=True),
        ),
    ]



from django import forms
from django.contrib import admin
from django.contrib.admin import widgets
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.http.response import HttpResponseBadRequest
from django.template.response import TemplateResponse
from django.utils.translation import gettext_lazy as _

from .. import settings as filer_settings
from ..models import Clipboard, Folder, FolderRoot, tools
from .tools import AdminContext, admin_url_params_encoded, popup_status


class NewFolderForm(forms.ModelForm):
    class Meta:
        model = Folder
        fields = ('name',)
        widgets = {
            'name': widgets.AdminTextInputWidget,
        }


@login_required
def make_folder(request, folder_id=None):
    if not folder_id:
        folder_id = request.GET.get('parent_id')
    if not folder_id:
        folder_id = request.POST.get('parent_id')
    if folder_id:
        try:
            folder = Folder.objects.get(id=folder_id)
        except Folder.DoesNotExist:
            raise PermissionDenied
    else:
        folder = None

    if request.user.is_superuser:
        pass
    elif folder is None:
        # regular users may not add root folders unless configured otherwise
        if not filer_settings.FILER_ALLOW_REGULAR_USERS_TO_ADD_ROOT_FOLDERS:
            raise PermissionDenied
    elif not folder.has_add_children_permission(request):
        # the user does not have the permission to add subfolders
        raise PermissionDenied

    if request.method == 'POST':
        new_folder_form = NewFolderForm(request.POST)
        if new_folder_form.is_valid():
            new_folder = new_folder_form.save(commit=False)
            if (folder or FolderRoot()).contains_folder(new_folder.name):
                new_folder_form._errors['name'] = new_folder_form.error_class(
                    [_('Folder with this name already exists.')])
            else:
                context = admin.site.each_context(request)
                new_folder.parent = folder
                new_folder.owner = request.user
                new_folder.save()
                return TemplateResponse(request, 'admin/filer/dismiss_popup.html', context)
    else:
        new_folder_form = NewFolderForm()

    context = admin.site.each_context(request)
    context.update({
        'opts': Folder._meta,
        'new_folder_form': new_folder_form,
        'is_popup': popup_status(request),
        'filer_admin_context': AdminContext(request),
    })
    return TemplateResponse(request, 'admin/filer/folder/new_folder_form.html', context)


@login_required
def paste_clipboard_to_folder(request):
    if True:
        # TODO: cleanly remove Clipboard code if it is no longer needed
        return HttpResponseBadRequest('not implemented anymore')

    if request.method == 'POST':
        folder = Folder.objects.get(id=request.POST.get('folder_id'))
        clipboard = Clipboard.objects.get(id=request.POST.get('clipboard_id'))
        if folder.has_add_children_permission(request):
            tools.move_files_from_clipboard_to_folder(clipboard, folder)
            tools.discard_clipboard(clipboard)
        else:
            raise PermissionDenied
    redirect = request.GET.get('redirect_to', '')
    if not redirect:
        redirect = request.POST.get('redirect_to', '')
    return HttpResponseRedirect(
        '{}?order_by=-modified_at{}'.format(
            redirect,
            admin_url_params_encoded(request, first_separator='&'),
        )
    )


@login_required
def discard_clipboard(request):
    if True:
        # TODO: cleanly remove Clipboard code if it is no longer needed
        return HttpResponseBadRequest('not implemented anymore')

    if request.method == 'POST':
        clipboard = Clipboard.objects.get(id=request.POST.get('clipboard_id'))
        tools.discard_clipboard(clipboard)
    return HttpResponseRedirect(
        '{}{}'.format(
            request.POST.get('redirect_to', ''),
            admin_url_params_encoded(request, first_separator='&'),
        )
    )


@login_required
def delete_clipboard(request):
    if True:
        # TODO: cleanly remove Clipboard code if it is no longer needed
        return HttpResponseBadRequest('not implemented anymore')

    if request.method == 'POST':
        clipboard = Clipboard.objects.get(id=request.POST.get('clipboard_id'))
        tools.delete_clipboard(clipboard)
    return HttpResponseRedirect(
        '{}{}'.format(
            request.POST.get('redirect_to', ''),
            admin_url_params_encoded(request, first_separator='&'),
        )
    )



import os
import shutil
import tempfile
from io import BytesIO, StringIO

from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.management import call_command
from django.test import TestCase
from django.utils.module_loading import import_string
from django.core.files.base import ContentFile

from filer import settings as filer_settings
from filer.models.filemodels import File
from filer.settings import FILER_IMAGE_MODEL
from filer.utils.loader import load_model
from tests.helpers import create_image

Image = load_model(FILER_IMAGE_MODEL)


class FilerCheckTestCase(TestCase):
    svg_file_string = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
    <!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "
    http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
    <svg version="1.1" baseProfile="full" width="50" height="50" xmlns="http://www.w3.org/2000/svg">
       <polygon id="triangle" points="0,0 0,50 50,0" fill="#009900"
    stroke="#004400"/>
       {}
    </svg>"""

    def setUp(self):
        # Clear all configured storages to ensure a clean state for each test.
        # This prevents interference from files left in any storage.
        for storage_alias, storage_configs in filer_settings.FILER_STORAGES.items():
            config = storage_configs.get('main')
            if not config:
                continue
            storage = import_string(config['ENGINE'])()
            upload_prefix = config.get('UPLOAD_TO_PREFIX', '')
            if storage.exists(upload_prefix):
                shutil.rmtree(storage.path(upload_prefix))

        # Create a sample file for testing in the public storage.
        original_filename = 'testimage.jpg'
        file_obj = SimpleUploadedFile(
            name=original_filename,
            content=create_image().tobytes(),
            content_type='image/jpeg'
        )
        self.filer_file = File.objects.create(
            file=file_obj,
            original_filename=original_filename
        )

    def tearDown(self):
        self.filer_file.delete()

    def test_delete_missing(self):
        out = StringIO()
        self.assertTrue(os.path.exists(self.filer_file.file.path))
        file_pk = self.filer_file.id
        call_command('filer_check', stdout=out, missing=True)
        self.assertEqual('', out.getvalue())

        # Remove the file to simulate a missing file.
        os.remove(self.filer_file.file.path)
        call_command('filer_check', stdout=out, missing=True)
        # When verbosity is low, a simple relative file path is output.
        self.assertEqual("None/testimage.jpg\n", out.getvalue())
        self.assertIsInstance(File.objects.get(id=file_pk), File)

        call_command('filer_check', delete_missing=True, interactive=False, verbosity=0)
        with self.assertRaises(File.DoesNotExist):
            File.objects.get(id=file_pk)

    def test_delete_orphans_public(self):
        # First check - should be empty initially
        out = StringIO()
        call_command('filer_check', stdout=out, orphans=True, verbosity=1)
        self.assertEqual('', out.getvalue())

        # Add an orphan file using the storage API directly
        public_settings = filer_settings.FILER_STORAGES['public']['main']
        storage = import_string(public_settings['ENGINE'])()

        # Configure storage location if specified in settings
        if public_settings.get('OPTIONS', {}).get('location'):
            storage.location = public_settings['OPTIONS']['location']

        # Get upload prefix and create file path
        prefix = public_settings.get('UPLOAD_TO_PREFIX', '')
        file_path = 'hello.txt'
        rel_path = os.path.join(prefix, file_path) if prefix else file_path

        # Save file through storage API
        storage.save(rel_path, ContentFile(b"I don't belong here!"))
        self.assertTrue(storage.exists(rel_path))

        # Check if orphan is detected
        out = StringIO()
        call_command('filer_check', stdout=out, orphans=True, verbosity=1)
        self.assertEqual("public/hello.txt\n", out.getvalue())

        # Delete orphans
        call_command('filer_check', delete_orphans=True, interactive=False, verbosity=0)
        self.assertFalse(storage.exists(rel_path))

    def test_delete_orphans_private(self):
        # Skip test if private storage is not configured.
        if 'private' not in filer_settings.FILER_STORAGES:
            self.skipTest("Private storage not configured in FILER_STORAGES.")

        out = StringIO()
        private_settings = filer_settings.FILER_STORAGES['private']['main']
        storage = import_string(private_settings['ENGINE'])()
        # Set storage location if defined in OPTIONS.
        if private_settings.get('OPTIONS', {}).get('location'):
            storage.location = private_settings['OPTIONS']['location']
        private_path = storage.path(private_settings.get('UPLOAD_TO_PREFIX', ''))
        os.makedirs(private_path, exist_ok=True)

        orphan_file = os.path.join(private_path, 'private_orphan.txt')
        with open(orphan_file, 'w') as fh:
            fh.write("I don't belong here!")
        # Run the command and check that it detects the private orphan file.
        call_command('filer_check', stdout=out, orphans=True)
        self.assertIn("private_orphan.txt", out.getvalue())
        self.assertTrue(os.path.exists(orphan_file))

        # Delete the orphan file.
        call_command('filer_check', delete_orphans=True, interactive=False, verbosity=0)
        self.assertFalse(os.path.exists(orphan_file))

    def test_delete_orphans_multiple_storages(self):
        """
        Test that the filer_check command correctly handles orphaned files in multiple storages
        without permanently modifying the settings. We use monkey-patching to assign temporary
        directories to the storage configurations.
        """
        out = StringIO()

        # --- Monkey-patch public storage location ---
        public_config = filer_settings.FILER_STORAGES['public']['main']
        temp_public_dir = tempfile.mkdtemp()
        if 'OPTIONS' in public_config:
            public_config['OPTIONS']['location'] = temp_public_dir
        else:
            public_config['OPTIONS'] = {'location': temp_public_dir}
        # Determine the upload prefix (if any) and ensure the corresponding directory exists.
        public_upload_prefix = public_config.get('UPLOAD_TO_PREFIX', '')
        if public_upload_prefix:
            public_full_dir = os.path.join(temp_public_dir, public_upload_prefix)
        else:
            public_full_dir = temp_public_dir
        os.makedirs(public_full_dir, exist_ok=True)

        # --- Monkey-patch private storage location ---
        private_config = filer_settings.FILER_STORAGES.get('private', {}).get('main')
        if private_config:
            temp_private_dir = tempfile.mkdtemp()
            if 'OPTIONS' in private_config:
                private_config['OPTIONS']['location'] = temp_private_dir
            else:
                private_config['OPTIONS'] = {'location': temp_private_dir}
            private_upload_prefix = private_config.get('UPLOAD_TO_PREFIX', '')
            if private_upload_prefix:
                private_full_dir = os.path.join(temp_private_dir, private_upload_prefix)
            else:
                private_full_dir = temp_private_dir
            os.makedirs(private_full_dir, exist_ok=True)
        else:
            self.skipTest("Private storage not configured in FILER_STORAGES.")

        # --- Initialize storages using the patched locations ---
        from django.core.files.storage import FileSystemStorage
        storage_public = FileSystemStorage(location=temp_public_dir)
        storage_private = FileSystemStorage(location=private_config['OPTIONS']['location'])

        # --- Save dummy orphan files in both storages ---
        # For public storage, include the upload prefix in the filename if needed.
        if public_upload_prefix:
            filename_public = os.path.join(public_upload_prefix, 'orphan_public.txt')
        else:
            filename_public = 'orphan_public.txt'
        if private_config.get('UPLOAD_TO_PREFIX', ''):
            filename_private = os.path.join(private_config['UPLOAD_TO_PREFIX'], 'orphan_private.txt')
        else:
            filename_private = 'orphan_private.txt'

        storage_public.save(filename_public, ContentFile(b"dummy content"))
        storage_private.save(filename_private, ContentFile(b"dummy content"))

        # --- Run the filer_check command ---
        call_command('filer_check', stdout=out, orphans=True)
        output = out.getvalue()

        # Verify that the output contains indicators for both storages.
        self.assertIn('public', output)
        self.assertIn('private', output)

        # --- Clean up ---
        storage_public.delete(filename_public)
        storage_private.delete(filename_private)
        shutil.rmtree(temp_public_dir)
        shutil.rmtree(private_config['OPTIONS']['location'])

    def test_image_dimensions_corrupted_file(self):
        original_filename = 'testimage.jpg'
        file_obj = SimpleUploadedFile(
            name=original_filename,
            content=create_image().tobytes(),  # Simulate a corrupted file.
            content_type='image/jpeg'
        )
        self.filer_image = Image.objects.create(
            file=file_obj,
            original_filename=original_filename
        )
        self.filer_image._width = 0
        self.filer_image.save()
        call_command('filer_check', image_dimensions=True)

    def test_image_dimensions_file_not_found(self):
        self.filer_image = Image.objects.create(
            file="123.jpg",
            original_filename="123.jpg"
        )
        call_command('filer_check', image_dimensions=True)
        self.filer_image.refresh_from_db()

    def test_image_dimensions(self):
        original_filename = 'testimage.jpg'
        with BytesIO() as jpg:
            create_image().save(jpg, format='JPEG')
            jpg.seek(0)
            file_obj = SimpleUploadedFile(
                name=original_filename,
                content=jpg.read(),
                content_type='image/jpeg'
            )
            self.filer_image = Image.objects.create(
                file=file_obj,
                original_filename=original_filename
            )
        self.filer_image._width = 0
        self.filer_image.save()

        call_command('filer_check', image_dimensions=True)
        self.filer_image.refresh_from_db()
        self.assertGreater(self.filer_image._width, 0)

    def test_image_dimensions_invalid_svg(self):
        original_filename = 'test.svg'
        svg_file = bytes("<asdva>" + self.svg_file_string, "utf-8")
        file_obj = SimpleUploadedFile(
            name=original_filename,
            content=svg_file,
            content_type='image/svg+xml'
        )
        self.filer_image = Image.objects.create(
            file=file_obj,
            original_filename=original_filename
        )
        self.filer_image._width = 0
        self.filer_image.save()
        call_command('filer_check', image_dimensions=True)

    def test_image_dimensions_svg(self):
        original_filename = 'test.svg'
        svg_file = bytes(self.svg_file_string, "utf-8")
        file_obj = SimpleUploadedFile(
            name=original_filename,
            content=svg_file,
            content_type='image/svg+xml'
        )
        self.filer_image = Image.objects.create(
            file=file_obj,
            original_filename=original_filename
        )
        self.filer_image._width = 0
        self.filer_image.save()

        call_command('filer_check', image_dimensions=True)
        self.filer_image.refresh_from_db()
        self.assertGreater(self.filer_image._width, 0)



import typing

from django.core.cache import cache
from django.db.models import Model


UserModel = typing.TypeVar('UserModel', bound=Model)


def get_folder_perm_cache_key(user: UserModel, permission: str) -> str:
    """
    Generates a unique cache key for a given user and permission.

    The key is a string in the format "filer:perm:<permission>", i.e. it does not
    contain the user id. This will be sufficient for most use cases.

    Patch this method to include the user id in the cache key if necessary, e.g.,
    for far more than 1,000 admin users to make the cached unit require less memory.

    Parameters:
    user (UserModel): The user for whom the cache key is being generated.
        The `user` can be an instance of the default `django.contrib.auth.models.User`
        or any custom user model specified by `AUTH_USER_MODEL` in the settings.
    permission (str): The permission for which the cache key is being generated.

    Returns:
    str: The generated cache key.
    """
    return f"filer:perm:{permission}"


def get_folder_permission_cache(user: UserModel, permission: str) -> typing.Optional[typing.Set[int]]:
    """
    Retrieves the cached folder permissions for a given user and permission.

    If the cache value exists, it returns the permissions for the user.
    If the cache value does not exist, it returns None.

    Parameters:
    user (UserModel): The user for whom the permissions are being retrieved.
        The `user` can be an instance of the default `django.contrib.auth.models.User`
        or any custom user model specified by `AUTH_USER_MODEL` in the settings.
    permission (str): The permission for which the permissions are being retrieved.

    Returns:
    set or None: The permissions for the user, or None if no cache value exists.
    """
    cache_value = cache.get(get_folder_perm_cache_key(user, permission))
    if cache_value:
        return cache_value.get(user.pk, None)
    return None


def clear_folder_permission_cache(user: UserModel, permission: typing.Optional[str] = None) -> None:
    """
    Clears the cached folder permissions for a given user.

    If a specific permission is provided, it clears the cache for that permission only.
    If no specific permission is provided, it clears the cache for all permissions.

    Parameters:
    user (UserModel): The user for whom the permissions are being cleared.
        The `user` can be an instance of the default `django.contrib.auth.models.User`
        or any custom user model specified by `AUTH_USER_MODEL` in the settings.
    permission (str, optional): The specific permission to clear. Defaults to None.
    """
    if permission is None:
        for perm in ['can_read', 'can_edit', 'can_add_children']:
            cache.delete(get_folder_perm_cache_key(user, perm))
    else:
        cache.delete(get_folder_perm_cache_key(user, permission))


def update_folder_permission_cache(user: UserModel, permission: str, id_list: typing.Set[int]) -> None:
    """
    Updates the cached folder permissions for a given user and permission.

    It first retrieves the current permissions from the cache (or an empty dictionary if none exist).
    Then it updates the permissions for the user with the provided list of IDs.
    Finally, it sets the updated permissions back into the cache.

    Parameters:
    user (UserModel): The user for whom the permissions are being updated.
        The `user` can be an instance of the default `django.contrib.auth.models.User`
        or any custom user model specified by `AUTH_USER_MODEL` in the settings.
    permission (str): The permission to update.
    id_list (set): The set of IDs to set as the new permissions.
    """
    perms = cache.get(get_folder_perm_cache_key(user, permission)) or {}
    perms[user.pk] = set(id_list)
    cache.set(get_folder_perm_cache_key(user, permission), perms)



from django.http import HttpResponse

from .base import ServerBase


class ApacheXSendfileServer(ServerBase):
    def serve(self, request, filer_file, **kwargs):
        response = HttpResponse()
        response['X-Sendfile'] = filer_file.path

        # This is needed for lighttpd, hopefully this will
        # not be needed after this is fixed:
        # http://redmine.lighttpd.net/issues/2076
        response['Content-Type'] = filer_file.mime_type

        self.default_headers(request=request, response=response, file_obj=filer_file.file, **kwargs)
        return response



from django import VERSION as django_version
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.urls import reverse


class PrimitivePermissionAwareModelAdmin(admin.ModelAdmin):
    def get_autocomplete_fields(self, request):
        """Remove "owner" from autocomplete_fields is User model has no search_fields"""

        autocomplete_fields = super().get_autocomplete_fields(request)
        if django_version >= (5, 0):
            user_admin = self.admin_site.get_model_admin(get_user_model())
        else:
            user_admin = self.admin_site._registry[get_user_model()]
        if not user_admin.get_search_fields(request) and 'owner' in autocomplete_fields:
            autocomplete_fields.remove('owner')
        return autocomplete_fields

    def has_add_permission(self, request):
        # we don't have a "add" permission... but all adding is handled
        # by special methods that go around these permissions anyway
        # TODO: reactivate return False
        return False

    def has_change_permission(self, request, obj=None):
        if hasattr(obj, 'has_edit_permission'):
            if obj.has_edit_permission(request):
                return True
            else:
                return False
        else:
            return True

    def has_delete_permission(self, request, obj=None):
        # we don't have a specific delete permission... so we use change
        return self.has_change_permission(request, obj)

    def _get_post_url(self, obj):
        """
        Needed to retrieve the changelist url as Folder/File can be extended
        and admin url may change
        """
        # Code from django ModelAdmin to determine changelist on the fly
        opts = obj._meta
        return reverse('admin:%s_%s_changelist' %
                       (opts.app_label, opts.model_name),
            current_app=self.admin_site.name)



# Generated by Django 1.11.7 on 2017-12-20 16:35
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0008_auto_20171117_1313'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='_file_size',
            field=models.BigIntegerField(blank=True, null=True, verbose_name='file size'),
        ),
    ]



import os

from django.conf import settings
from django.contrib.admin import helpers
from django.contrib.auth.models import Group, Permission
from django.core.cache import cache
from django.core.files import File as DjangoFile
from django.test.testcases import TestCase
from django.urls import reverse

from filer import settings as filer_settings
from filer.models.clipboardmodels import Clipboard
from filer.models.foldermodels import Folder, FolderPermission
from filer.settings import FILER_IMAGE_MODEL
from filer.utils.loader import load_model
from tests.helpers import create_image, create_superuser


Image = load_model(FILER_IMAGE_MODEL)


class Mock():
    pass


class FolderPermissionsTestCase(TestCase):

    def setUp(self):
        from django.contrib.auth import get_user_model
        User = get_user_model()

        self.superuser = create_superuser()
        self.client.login(username='admin', password='secret')

        self.unauth_user = User.objects.create(username='unauth_user')

        self.owner = User.objects.create(username='owner')

        perms = Permission.objects.filter(codename="change_folder")
        perms |= Permission.objects.filter(codename="can_use_directory_listing")
        self.test_user1 = User.objects.create(username='test1', password='secret', is_staff=True)
        self.test_user1.set_password("secret")
        self.test_user1.save()
        self.test_user2 = User.objects.create(username='test2', password='secret')
        self.test_user1.user_permissions.add(*perms)
        self.test_user2.user_permissions.add(*perms)

        self.group1 = Group.objects.create(name='name1')
        self.group2 = Group.objects.create(name='name2')

        self.test_user1.groups.add(self.group1)
        self.test_user2.groups.add(self.group2)

        self.img = create_image()
        self.image_name = 'test_file.jpg'
        self.filename = os.path.join(settings.FILE_UPLOAD_TEMP_DIR, self.image_name)
        self.img.save(self.filename, 'JPEG')

        self.file = DjangoFile(open(self.filename, 'rb'), name=self.image_name)
        # This is actually a "file" for filer considerations
        self.image = Image.objects.create(owner=self.superuser,
                                     original_filename=self.image_name,
                                     file=self.file)
        self.clipboard = Clipboard.objects.create(user=self.superuser)
        self.clipboard.append_file(self.image)

        self.folder = Folder.objects.create(name='test_folder')

        self.folder_perm = Folder.objects.create(name='test_folder2')

    def tearDown(self):
        self.file.close()
        self.image.delete()

    def test_folder_all(self):
        folder = Folder.objects.create(name="test_folder3")
        self.assertEqual(str(folder), "/test_folder3")

    def test_superuser_has_rights(self):
        request = Mock()
        setattr(request, 'user', self.superuser)

        result = self.folder.has_read_permission(request)
        self.assertEqual(result, True)

    def test_unlogged_user_has_no_rights(self):
        old_setting = filer_settings.FILER_ENABLE_PERMISSIONS
        try:
            filer_settings.FILER_ENABLE_PERMISSIONS = True
            request = Mock()
            setattr(request, 'user', self.unauth_user)

            result = self.folder.has_read_permission(request)
            self.assertEqual(result, False)
        finally:
            filer_settings.FILER_ENABLE_PERMISSIONS = old_setting

    def test_user_cannot_see_other_users_files(self):
        old_setting = filer_settings.FILER_ENABLE_PERMISSIONS
        try:
            filer_settings.FILER_ENABLE_PERMISSIONS = True

            url = reverse("admin:filer-directory_listing-unfiled_images")
            response = self.client.get(url)
            self.assertContains(response, self.image_name)
            loginok = self.client.login(username=self.test_user1.username, password="secret")
            self.assertTrue(loginok, "Could not login 'test_user1'")
            response = self.client.get(url)
            self.assertNotContains(response, self.image_name)

        finally:
            filer_settings.FILER_ENABLE_PERMISSIONS = old_setting

    def test_unlogged_user_has_rights_when_permissions_disabled(self):
        request = Mock()
        setattr(request, 'user', self.unauth_user)

        result = self.folder.has_read_permission(request)
        self.assertEqual(result, True)

    def test_owner_user_has_rights(self):
        # Set owner as the owner of the folder.
        self.folder.owner = self.owner
        request = Mock()
        setattr(request, 'user', self.owner)

        result = self.folder.has_read_permission(request)
        self.assertEqual(result, True)

    def test_combined_groups(self):
        request1 = Mock()
        setattr(request1, 'user', self.test_user1)
        request2 = Mock()
        setattr(request2, 'user', self.test_user2)

        old_setting = filer_settings.FILER_ENABLE_PERMISSIONS
        try:
            filer_settings.FILER_ENABLE_PERMISSIONS = True

            self.assertEqual(self.folder.has_read_permission(request1), False)
            self.assertEqual(self.folder.has_read_permission(request2), False)
            self.assertEqual(self.folder_perm.has_read_permission(request1), False)
            self.assertEqual(self.folder_perm.has_read_permission(request2), False)

            self.assertEqual(FolderPermission.objects.count(), 0)

            FolderPermission.objects.create(folder=self.folder, type=FolderPermission.CHILDREN, group=self.group1, can_edit=FolderPermission.DENY, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.DENY)
            FolderPermission.objects.create(folder=self.folder_perm, type=FolderPermission.CHILDREN, group=self.group2, can_edit=FolderPermission.DENY, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.DENY)

            self.assertEqual(FolderPermission.objects.count(), 2)

            # We have to invalidate cache
            delattr(self.folder, 'permission_cache')
            delattr(self.folder_perm, 'permission_cache')

            self.assertEqual(self.folder.has_read_permission(request1), True)
            self.assertEqual(self.folder.has_read_permission(request2), False)
            self.assertEqual(self.folder_perm.has_read_permission(request1), False)
            self.assertEqual(self.folder_perm.has_read_permission(request2), True)

            self.test_user1.groups.add(self.group2)
            self.test_user2.groups.add(self.group1)

            # We have to invalidate cache
            delattr(self.folder, 'permission_cache')
            delattr(self.folder_perm, 'permission_cache')
            cache.clear()

            self.assertEqual(self.folder.has_read_permission(request1), True)
            self.assertEqual(self.folder.has_read_permission(request2), True)
            self.assertEqual(self.folder_perm.has_read_permission(request1), True)
            self.assertEqual(self.folder_perm.has_read_permission(request2), True)

            self.assertEqual(self.folder_perm.pretty_logical_path, "/test_folder2")
        finally:
            filer_settings.FILER_ENABLE_PERMISSIONS = old_setting
            cache.clear()

    def test_overlapped_groups_deny1(self):
        # Tests overlapped groups with explicit deny

        request1 = Mock()
        setattr(request1, 'user', self.test_user1)

        old_setting = filer_settings.FILER_ENABLE_PERMISSIONS
        try:
            filer_settings.FILER_ENABLE_PERMISSIONS = True

            self.assertEqual(self.folder.has_read_permission(request1), False)
            self.assertEqual(self.folder_perm.has_read_permission(request1), False)

            self.assertEqual(FolderPermission.objects.count(), 0)

            FolderPermission.objects.create(folder=self.folder, type=FolderPermission.CHILDREN, group=self.group1, can_edit=FolderPermission.DENY, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.DENY)
            FolderPermission.objects.create(folder=self.folder, type=FolderPermission.CHILDREN, group=self.group2, can_edit=FolderPermission.ALLOW, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.ALLOW)

            self.assertEqual(FolderPermission.objects.count(), 2)

            # We have to invalidate cache
            delattr(self.folder, 'permission_cache')

            self.assertEqual(self.test_user1.groups.filter(pk=self.group1.pk).exists(), True)
            self.assertEqual(self.test_user1.groups.filter(pk=self.group2.pk).exists(), False)

            self.assertEqual(self.folder.has_read_permission(request1), True)
            self.assertEqual(self.folder.has_edit_permission(request1), False)

            self.assertEqual(self.test_user1.groups.count(), 1)

            self.test_user1.groups.add(self.group2)

            self.assertEqual(self.test_user1.groups.count(), 2)

            # We have to invalidate cache
            delattr(self.folder, 'permission_cache')

            self.assertEqual(self.folder.has_read_permission(request1), True)
            self.assertEqual(self.folder.has_edit_permission(request1), False)

        finally:
            filer_settings.FILER_ENABLE_PERMISSIONS = old_setting
            cache.clear()

    def test_overlapped_groups_deny2(self):
        # Tests overlapped groups with explicit deny
        # Similar test to test_overlapped_groups_deny1, only order of groups is different

        request2 = Mock()
        setattr(request2, 'user', self.test_user2)

        old_setting = filer_settings.FILER_ENABLE_PERMISSIONS
        try:
            filer_settings.FILER_ENABLE_PERMISSIONS = True

            self.assertEqual(self.folder.has_read_permission(request2), False)
            self.assertEqual(self.folder_perm.has_read_permission(request2), False)

            self.assertEqual(FolderPermission.objects.count(), 0)

            FolderPermission.objects.create(folder=self.folder_perm, type=FolderPermission.CHILDREN, group=self.group2, can_edit=FolderPermission.DENY, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.DENY)
            FolderPermission.objects.create(folder=self.folder_perm, type=FolderPermission.CHILDREN, group=self.group1, can_edit=FolderPermission.ALLOW, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.ALLOW)

            self.assertEqual(FolderPermission.objects.count(), 2)

            # We have to invalidate cache
            delattr(self.folder_perm, 'permission_cache')

            self.assertEqual(self.test_user2.groups.filter(pk=self.group2.pk).exists(), True)
            self.assertEqual(self.test_user2.groups.filter(pk=self.group1.pk).exists(), False)

            self.assertEqual(self.folder_perm.has_read_permission(request2), True)
            self.assertEqual(self.folder_perm.has_edit_permission(request2), False)

            self.assertEqual(self.test_user2.groups.count(), 1)

            self.test_user2.groups.add(self.group1)

            self.assertEqual(self.test_user2.groups.count(), 2)

            # We have to invalidate cache
            delattr(self.folder_perm, 'permission_cache')

            self.assertEqual(self.folder_perm.has_read_permission(request2), True)
            self.assertEqual(self.folder_perm.has_edit_permission(request2), False)

        finally:
            filer_settings.FILER_ENABLE_PERMISSIONS = old_setting
            cache.clear()

    def test_overlapped_groups1(self):
        # Tests overlapped groups without explicit deny

        request1 = Mock()
        setattr(request1, 'user', self.test_user1)

        old_setting = filer_settings.FILER_ENABLE_PERMISSIONS
        try:
            filer_settings.FILER_ENABLE_PERMISSIONS = True

            self.assertEqual(self.folder.has_read_permission(request1), False)
            self.assertEqual(self.folder_perm.has_read_permission(request1), False)

            self.assertEqual(FolderPermission.objects.count(), 0)

            FolderPermission.objects.create(folder=self.folder, type=FolderPermission.CHILDREN, group=self.group1, can_edit=None, can_read=FolderPermission.ALLOW, can_add_children=None)
            FolderPermission.objects.create(folder=self.folder, type=FolderPermission.CHILDREN, group=self.group2, can_edit=FolderPermission.ALLOW, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.ALLOW)

            self.assertEqual(FolderPermission.objects.count(), 2)

            # We have to invalidate cache
            delattr(self.folder, 'permission_cache')

            self.assertEqual(self.test_user1.groups.filter(pk=self.group1.pk).exists(), True)
            self.assertEqual(self.test_user1.groups.filter(pk=self.group2.pk).exists(), False)

            self.assertEqual(self.folder.has_read_permission(request1), True)
            self.assertEqual(self.folder.has_edit_permission(request1), False)

            self.assertEqual(self.test_user1.groups.count(), 1)

            self.test_user1.groups.add(self.group2)

            self.assertEqual(self.test_user1.groups.count(), 2)

            # We have to invalidate cache
            delattr(self.folder, 'permission_cache')

            self.assertEqual(self.folder.has_read_permission(request1), True)
            self.assertEqual(self.folder.has_edit_permission(request1), True)

        finally:
            filer_settings.FILER_ENABLE_PERMISSIONS = old_setting
            cache.clear()

    def test_overlapped_groups2(self):
        # Tests overlapped groups without explicit deny
        # Similar test to test_overlapped_groups1, only order of groups is different

        request2 = Mock()
        setattr(request2, 'user', self.test_user2)

        old_setting = filer_settings.FILER_ENABLE_PERMISSIONS
        try:
            filer_settings.FILER_ENABLE_PERMISSIONS = True

            self.assertEqual(self.folder.has_read_permission(request2), False)
            self.assertEqual(self.folder_perm.has_read_permission(request2), False)

            self.assertEqual(FolderPermission.objects.count(), 0)

            FolderPermission.objects.create(folder=self.folder_perm, type=FolderPermission.CHILDREN, group=self.group2, can_edit=None, can_read=FolderPermission.ALLOW, can_add_children=None)
            FolderPermission.objects.create(folder=self.folder_perm, type=FolderPermission.CHILDREN, group=self.group1, can_edit=FolderPermission.ALLOW, can_read=FolderPermission.ALLOW, can_add_children=FolderPermission.ALLOW)

            self.assertEqual(FolderPermission.objects.count(), 2)

            # We have to invalidate cache
            delattr(self.folder_perm, 'permission_cache')

            self.assertEqual(self.test_user2.groups.filter(pk=self.group2.pk).exists(), True)
            self.assertEqual(self.test_user2.groups.filter(pk=self.group1.pk).exists(), False)

            self.assertEqual(self.folder_perm.has_read_permission(request2), True)
            self.assertEqual(self.folder_perm.has_edit_permission(request2), False)

            self.assertEqual(self.test_user2.groups.count(), 1)

            self.test_user2.groups.add(self.group1)

            self.assertEqual(self.test_user2.groups.count(), 2)

            # We have to invalidate cache
            delattr(self.folder_perm, 'permission_cache')

            self.assertEqual(self.folder_perm.has_read_permission(request2), True)
            self.assertEqual(self.folder_perm.has_edit_permission(request2), True)

        finally:
            filer_settings.FILER_ENABLE_PERMISSIONS = old_setting
            cache.clear()

    def test_folder_who_owner(self):
        perm = FolderPermission.objects.create(
            user=self.owner,
            folder=Folder.objects.create(name="folder4"),
            type=FolderPermission.CHILDREN,
            can_edit=FolderPermission.DENY,
            can_read=FolderPermission.ALLOW,
            can_add_children=FolderPermission.ALLOW,
        )
        self.assertEqual(perm.who, "User: owner")
        self.assertEqual(perm.pretty_logical_path, "/folder4")
        self.assertEqual(perm.what, "E̶d̶i̶t̶, Read, Add children")

    def test_folder_who_group(self):
        perm = FolderPermission.objects.create(
            group=self.group1,
        )
        self.assertEqual(perm.who, "Group: name1")

    def test_folder_who_everybody(self):
        perm = FolderPermission.objects.create(
            everybody=True,
        )
        self.assertEqual(perm.who, "Everybody")
        self.assertEqual(perm.pretty_logical_path, "All Folders")
        self.assertEqual(perm.__str__(), "All Folders")

    def test_folder_who_nobody(self):
        perm = FolderPermission.objects.create()
        self.assertEqual(perm.who, "–")

    def test_folderpermission_is_copied(self):
        source_folder = Folder.objects.create(name="source")
        destination_folder = Folder.objects.create(name="destination")
        perm = FolderPermission.objects.create(
            user=self.owner,
            folder=source_folder,
            type=FolderPermission.CHILDREN,
            can_edit=FolderPermission.ALLOW,
            can_read=FolderPermission.DENY,
            can_add_children=FolderPermission.ALLOW,
        )
        self.assertEqual(FolderPermission.objects.count(), 1)
        url = reverse('admin:filer-directory_listing-root')
        self.client.post(url, {
            'action': 'copy_files_and_folders',
            'post': 'yes',
            'suffix': 'test',
            'destination': destination_folder.id,
            helpers.ACTION_CHECKBOX_NAME: 'folder-%d' % (source_folder.id,),
        })
        self.assertEqual(FolderPermission.objects.count(), 2)
        copied_folder = destination_folder.children.first()
        self.assertEqual(copied_folder.name, source_folder.name)
        new_perm = FolderPermission.objects.get(folder=copied_folder)
        self.assertEqual(perm.user, new_perm.user)
        self.assertEqual(perm.type, new_perm.type)
        self.assertEqual(perm.can_edit, new_perm.can_edit)
        self.assertEqual(perm.can_read, new_perm.can_read)
        self.assertEqual(perm.can_add_children, new_perm.can_add_children)



from math import ceil

from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.files.storage import FileSystemStorage
from django.template import Library
from django.templatetags.static import static
from django.urls import reverse
from django.utils.html import escapejs, format_html_join
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from easy_thumbnails.engine import NoSourceGenerator
from easy_thumbnails.exceptions import InvalidImageFormatError
from easy_thumbnails.files import get_thumbnailer
from easy_thumbnails.options import ThumbnailOptions

from filer import settings
from filer.admin.tools import admin_url_params, admin_url_params_encoded
from filer.models.imagemodels import BaseImage
from filer.settings import (
    DEFERRED_THUMBNAIL_SIZES, FILER_MAX_SVG_THUMBNAIL_SIZE, FILER_TABLE_ICON_SIZE, FILER_THUMBNAIL_ICON_SIZE,
)


register = Library()

assignment_tag = getattr(register, 'assignment_tag', register.simple_tag)


def filer_actions(context):
    """
    Track the number of times the action field has been rendered on the page,
    so we know which value to use.
    """
    context['action_index'] = context.get('action_index', -1) + 1
    return context


filer_actions = register.inclusion_tag(
    "admin/filer/actions.html", takes_context=True)(filer_actions)


@register.inclusion_tag(
    'admin/filer/folder/list_type_switcher.html',
    takes_context=True,
)
def filer_folder_list_type_switcher(context):
    choice_list = settings.FILER_FOLDER_ADMIN_LIST_TYPE_CHOICES
    current_list_type = context['list_type']
    # This solution is user-friendly when there's only 2 choices
    # If there will be more list types then please change this switcher to more
    # proper widget (like for e.g. select list)
    next_list_type = next(
        iter(choice_list[choice_list.index(current_list_type) + 1:]),
        choice_list[0],
    )

    context['url'] = admin_url_params_encoded(
        context['request'],
        params={'_list_type': next_list_type},
    )
    context['tooltip_text'] = settings.FILER_FOLDER_ADMIN_LIST_TYPE_SWITCHER_SETTINGS[next_list_type]['tooltip_text']
    context['icon'] = settings.FILER_FOLDER_ADMIN_LIST_TYPE_SWITCHER_SETTINGS[next_list_type]['icon']
    return context


@register.simple_tag(takes_context=True)
def filer_admin_context_url_params(context, first_separator='?'):
    return admin_url_params_encoded(
        context['request'], first_separator=first_separator)


@register.simple_tag(takes_context=True)
def filer_admin_context_hidden_formfields(context):
    request = context.get('request')
    return format_html_join(
        '\n',
        '<input type="hidden" name="{0}" value="{1}">',
        admin_url_params(request).items(),
    )


@assignment_tag(takes_context=True)
def filer_has_permission(context, item, action):
    """Does the current user (taken from the request in the context) have
    permission to do the given action on the given item.

    """
    permission_method_name = f'has_{action}_permission'
    permission_method = getattr(item, permission_method_name, None)
    request = context.get('request')

    if not permission_method or not request:
        return False
    # Call the permission method.
    # This amounts to calling `item.has_X_permission(request)`
    return permission_method(request)


def file_icon_context(file, detail, width, height):
    mime_maintype, mime_subtype = file.mime_maintype, file.mime_subtype
    context = {
        'mime_maintype': mime_maintype,
        'mime_type': file.mime_type,
    }
    height, width, context = get_aspect_ratio_and_download_url(context, detail, file, height, width)
    # returned context if icon is not available
    not_available_context = {
        'icon_url': staticfiles_storage.url('filer/icons/file-missing.svg'),
        'alt_text': _("File is missing"),
        'width': width,
        'height': width,  # The icon is a square
    }
    # Check if file exists for performance reasons (only on FileSystemStorage)
    if file.file and isinstance(file.file.source_storage, FileSystemStorage) and not file.file.exists():
        return not_available_context

    add_styles = {}
    if isinstance(file, BaseImage):
        thumbnailer = get_thumbnailer(file)

        # SVG files may contain multiple vector graphics, and width and height are not available for them. If file does
        # not have width or height just ignore the thumbnail icon. Otherwise, continue with the standard procedure.
        if file.width == 0.0 or file.height == 0.0:
            icon_url = staticfiles_storage.url('filer/icons/file-unknown.svg')
        else:
            if detail:
                opts = {'size': (width, height), 'upscale': True}
            else:
                opts = {'size': (width, height), 'crop': True}
            thumbnail_options = ThumbnailOptions(opts)
            # Optimize directory listing:
            if width == height and width in DEFERRED_THUMBNAIL_SIZES and hasattr(file, "thumbnail_name"):
                # Get name of thumbnail from easy-thumbnail
                configured_name = thumbnailer.get_thumbnail_name(thumbnail_options, transparent=file._transparent)
                # If the name was annotated: Thumbnail exists and we can use it
                if configured_name == file.thumbnail_name:
                    icon_url = file.file.thumbnail_storage.url(configured_name)
                    if mime_subtype != 'svg+xml' and file.thumbnailx2_name:
                        context['highres_url'] = file.file.thumbnail_storage.url(file.thumbnailx2_name)
                elif mime_subtype == 'svg+xml':
                    if file.size < FILER_MAX_SVG_THUMBNAIL_SIZE:
                        icon_url = file.url
                    else:
                        # Only display a generic image icon
                        # Better solution: Render a bitmap thumbnail from SVG
                        icon_url = staticfiles_storage.url('filer/icons/file-picture.svg')
                    add_styles = {
                        "object-fit": "cover",  # Mimic the behavior of the thumbnail
                    }
                else:  # Probably does not exist, defer creation
                    icon_url = reverse("admin:filer_file_fileicon", args=(file.pk, width))
                context['alt_text'] = file.default_alt_text
            else:
                # Try creating thumbnails / take existing ones
                try:
                    icon_url = thumbnailer.get_thumbnail(thumbnail_options).url
                    context['alt_text'] = file.default_alt_text
                    if mime_subtype != 'svg+xml':
                        thumbnail_options['size'] = 2 * width, 2 * height
                        context['highres_url'] = thumbnailer.get_thumbnail(thumbnail_options).url
                except (InvalidImageFormatError, NoSourceGenerator):
                    # This is caught by file.exists() for file storage systems
                    # For remote storage systems we catch the error to avoid second trip
                    # to the storage system
                    return not_available_context
    elif mime_maintype in ['audio', 'font', 'video']:
        icon_url = staticfiles_storage.url(f'filer/icons/file-{mime_maintype}.svg')
        height = width  # icon is a square
    elif mime_maintype == 'application' and mime_subtype in ['zip', 'pdf']:
        icon_url = staticfiles_storage.url(f'filer/icons/file-{mime_subtype}.svg')
        height = width  # icon is a square
    else:
        icon_url = staticfiles_storage.url('filer/icons/file-unknown.svg')
        height = width  # icon is a square
    context.update(width=width, height=height, icon_url=icon_url)
    if add_styles:
        styles = "; ".join(f"{k}: {v}" for k, v in add_styles.items())
        context.update(add_styles=f'style="{styles}"')
    return context


def get_aspect_ratio_and_download_url(context, detail, file, height, width):
    # Get download_url and aspect ratio right for detail view
    if detail:
        context['download_url'] = file.url
        if isinstance(file, BaseImage):
            # only check for file width, if the file
            # is actually an image and not on other files
            # because they don't really have width or height
            if file.width:
                width, height = 210, ceil(210 / file.width * file.height)
                context['sidebar_image_ratio'] = '%.6f' % (file.width / 210)
    return height, width, context


@register.inclusion_tag('admin/filer/templatetags/file_icon.html')
def file_icon(file, detail=False, size=None):
    """
    This templatetag returns a rendered `<img src="..." srcset="..." width="..." height="..." class="..." />
    to be used for rendering thumbnails of files in the directory listing or in the corresponding detail
    views for that image.
    """
    if size:
        width, height = (int(s) for s in size.split('x'))
    elif detail == "thumbnail":
        width, height = FILER_THUMBNAIL_ICON_SIZE, FILER_THUMBNAIL_ICON_SIZE
    elif detail is True:
        width, height = 2 * FILER_TABLE_ICON_SIZE, 2 * FILER_TABLE_ICON_SIZE
    else:
        width, height = FILER_TABLE_ICON_SIZE, FILER_TABLE_ICON_SIZE
    return file_icon_context(file, detail is True, width, height)


@register.simple_tag
def file_icon_url(file):
    # Cache since it is called repeatedly by templates
    if not hasattr(file, "_file_icon_url_cache"):
        context = file_icon_context(file, False, 2 * FILER_TABLE_ICON_SIZE, 2 * FILER_TABLE_ICON_SIZE)
        file._file_icon_url_cache = escapejs(context.get('highres_url', context['icon_url']))
    return file._file_icon_url_cache


@register.simple_tag
def icon_css_library():
    html = ""
    for lib in settings.ICON_CSS_LIB:
        html += f'<link rel="stylesheet" type="text/css" href="{static(lib)}">'
    return mark_safe(html)



# Generated by Django 1.9.13 on 2017-11-17 13:13
from django.db import migrations, models

import filer.models.filemodels


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0007_auto_20161016_1055'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='is_public',
            field=models.BooleanField(default=filer.models.filemodels.is_public_default, help_text='Disable any permission checking for this file. File will be publicly accessible to anyone.', verbose_name='Permissions disabled'),
        ),
    ]



from django.utils.functional import keep_lazy
from django.utils.text import Truncator, format_lazy


def string_concat(*strings):
    return format_lazy('{}' * len(strings), *strings)


def truncate_words(s, num, end_text='...'):
    # truncate_words was removed in Django 1.5.
    truncate = end_text and ' %s' % end_text or ''
    return Truncator(s).words(num, truncate=truncate)


truncate_words = keep_lazy(truncate_words, str)


def get_delete_permission(opts):
    from django.contrib.auth import get_permission_codename
    return '{}.{}'.format(opts.app_label, get_permission_codename('delete', opts))


try:
    from PIL import ExifTags as PILExifTags
    from PIL import Image as PILImage
    from PIL import ImageDraw as PILImageDraw
except ImportError:
    try:
        import ExifTags as PILExifTags  # noqa
        import Image as PILImage  # noqa
        import ImageDraw as PILImageDraw  # noqa
    except ImportError:
        raise ImportError("The Python Imaging Library was not found.")



import json
import os

import django.core
from django.apps import apps
from django.conf import settings
from django.test import TestCase
from django.urls import reverse
from django.utils.crypto import get_random_string

from filer.models import File, Folder
from filer.validation import FileValidationError, validate_upload, sanitize_svg
from tests.helpers import create_superuser


class TestValidators(TestCase):

    def setUp(self) -> None:
        self.superuser = create_superuser()
        self.client.login(username='admin', password='secret')
        self.folder = Folder.objects.create(name='foo')

    def tearDown(self) -> None:
        self.folder.delete()

    svg_file = """<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "
http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
<svg version="1.1" baseProfile="full" width="50" height="50" xmlns="http://www.w3.org/2000/svg">
   <polygon id="triangle" points="0,0 0,50 50,0" fill="#009900"
stroke="#004400"/>
   {}
</svg>"""

    def test_html_upload_fails(self):
        html_file = 'test_file.html'
        filename = os.path.join(
            settings.FILE_UPLOAD_TEMP_DIR,
            html_file
        )

        with open(filename, 'wb') as fh:
            fh.write(b"<html><script>alert('hello filer');</script></html>")
        self.assertEqual(File.objects.count(), 0)

        with open(filename, 'rb') as fh:
            file_obj = django.core.files.File(fh)
            url = reverse('admin:filer-ajax_upload', kwargs={'folder_id': self.folder.pk})
            post_data = {
                'Filename': html_file,
                'Filedata': file_obj,
                'jsessionid': self.client.session.session_key
            }
            response = self.client.post(url, post_data)

        self.assertContains(response, "HTML upload denied by site security policy")
        self.assertEqual(File.objects.count(), 0)

    def test_svg_upload_fails(self):
        for attack, expected_files in [
            ("""<a href="javascript: alert('ing');">test</a>""", 0),
            ('<script>alert(document.domain);</script>', 0),
            ("""<circle onclick="console.log('test')" cx="300" cy="225" r="100" fill="red"/>""", 0),
            ("", 1)
        ]:
            svg_file = 'test_file.svg'
            filename = os.path.join(
                settings.FILE_UPLOAD_TEMP_DIR,
                svg_file
            )

            # create svg file with attack vector
            with open(filename, 'w') as fh:
                fh.write(self.svg_file.format(attack))
            n = File.objects.count()

            with open(filename, 'rb') as fh:
                file_obj = django.core.files.File(fh)
                url = reverse('admin:filer-ajax_upload', kwargs={'folder_id': self.folder.pk})
                post_data = {
                    'Filename': svg_file,
                    'Filedata': file_obj,
                    'jsessionid': self.client.session.session_key
                }
                response = self.client.post(url, post_data)
            if expected_files == 0:
                self.assertContains(response, "Rejected due to potential cross site scripting vulnerability")
            self.assertEqual(File.objects.count(), n + expected_files)

    def test_deny_validator(self):
        from filer.validation import deny

        self.assertRaisesRegex(
            FileValidationError,
            "HTML upload denied by site security policy",
            deny,
            "test.html",
            None,
            None,
            "text/html",
        )

        self.assertRaisesRegex(
            FileValidationError,
            "MY_FUNNY_EXT upload denied by site security policy",
            deny,
            "test.my_funny_ext",
            None,
            None,
            "text/html",
        )

        self.assertRaisesRegex(
            FileValidationError,
            "Upload denied by site security policy",
            deny,
            "test",
            None,
            None,
            "text/html",
        )

    def test_svg_sanitizer(self):
        config = apps.get_app_config("filer")
        svg_validation = config.FILE_VALIDATORS["image/svg+xml"]
        config.FILE_VALIDATORS["image/svg+xml"] = [sanitize_svg]
        for attack, disallowed in [
            ("""<a href="javascript: alert('ing');">test</a>""", "javascript:"),
            ('<script>alert(document.domain);</script>', "alert"),
            ("""<circle onclick="console.log('test')" cx="300" cy="225" r="100" fill="red"/>""", "onclick"),
        ]:
            svg_file = 'test_file.svg'
            filename = os.path.join(
                settings.FILE_UPLOAD_TEMP_DIR,
                svg_file
            )

            # create svg file with attack vector
            with open(filename, 'w') as fh:
                fh.write(self.svg_file.format(attack))

            with open(filename, 'rb') as fh:
                file_obj = django.core.files.File(fh)
                url = reverse('admin:filer-ajax_upload', kwargs={'folder_id': self.folder.pk})
                post_data = {
                    'Filename': svg_file,
                    'Filedata': file_obj,
                    'jsessionid': self.client.session.session_key
                }
                response = self.client.post(url, post_data)
            file_id = json.loads(response.content.decode("utf-8"))["file_id"]
            img = File.objects.get(pk=file_id)
            content = img.file.file.read().decode("utf-8")
            self.assertNotIn(disallowed, content)

        config.FILE_VALIDATORS["image/svg+xml"] = svg_validation


class TestWhitelist(TestCase):
    def setUp(self) -> None:
        self.superuser = create_superuser()
        self.client.login(username='admin', password='secret')
        self.folder = Folder.objects.create(name='foo')
        self.config = apps.get_app_config("filer")
        self.MIME_TYPE_WHITELIST = self.config.MIME_TYPE_WHITELIST

    def tearDown(self) -> None:
        self.folder.delete()
        self.config.MIME_TYPE_WHITELIST = self.MIME_TYPE_WHITELIST

    def set_whitelist(self, whitelist):
        self.config.MIME_TYPE_WHITELIST = whitelist

    def test_no_whitelist(self):
        self.set_whitelist([])
        for i in range(10):
            mime_type = get_random_string(6) + "/" + get_random_string(5)

            # If this throws an error, the test fails
            validate_upload(f"test.{mime_type.split('/')[-1]}", None, None, mime_type)

    def test_whitelist(self):
        self.set_whitelist(["text/*", "image/x-png"])

        expectation = {
            "text/plain": "ok",
            "text/html": "fail",  # OK by whitelist but blocked by html validator
            "image/x-png": "ok",
            "image/jpeg": "fail",
        }

        for mime_type, expected_result in expectation.items():
            if expected_result == "ok":
                try:
                    validate_upload("test-file", None, None, mime_type)
                except FileValidationError:
                    self.assertFalse(f"Mime type {mime_type} expected to pass")
            else:
                with self.assertRaises(FileValidationError):
                    validate_upload("test-file", None, None, mime_type)



import logging
from datetime import datetime

from django.conf import settings
from django.db import models
from django.utils.timezone import get_current_timezone, make_aware, now
from django.utils.translation import gettext_lazy as _

from .abstract import BaseImage


logger = logging.getLogger("filer")


# This is the standard Image model which can be swapped for a custom model using FILER_IMAGE_MODEL setting
class Image(BaseImage):
    date_taken = models.DateTimeField(
        _("date taken"),
        null=True,
        blank=True,
        editable=False,
    )

    author = models.CharField(
        _("author"),
        max_length=255,
        null=True,
        blank=True,
    )

    must_always_publish_author_credit = models.BooleanField(
        _("must always publish author credit"),
        default=False,
    )

    must_always_publish_copyright = models.BooleanField(
        _('must always publish copyright'),
        default=False,
    )

    class Meta(BaseImage.Meta):
        swappable = 'FILER_IMAGE_MODEL'
        default_manager_name = 'objects'

    def save(self, *args, **kwargs):
        if self.date_taken is None:
            try:
                exif_date = self.exif.get('DateTimeOriginal', None)
                if exif_date is not None:
                    d, t = exif_date.split(" ")
                    year, month, day = d.split(':')
                    hour, minute, second = t.split(':')
                    if getattr(settings, "USE_TZ", False):
                        tz = get_current_timezone()
                        self.date_taken = make_aware(datetime(
                            int(year), int(month), int(day),
                            int(hour), int(minute), int(second)), tz)
                    else:
                        self.date_taken = datetime(
                            int(year), int(month), int(day),
                            int(hour), int(minute), int(second))
            except Exception:
                pass
        if self.date_taken is None:
            self.date_taken = now()
        super().save(*args, **kwargs)
